--[[
Name: LibXML-1.0
Revision: $Rev: 11 $
Author(s): Humbedooh
Description: XML import library
Dependencies: LibStub
License: MIT License
]]
local LIBXML = "LibXML-1.0"
local LIBXML_MINOR = tonumber(("$Rev: 011 $"):match("(%d+)")) or 10000;
if not LibStub then error(LIBXML .. " requires LibStub.") end
local LibXML = LibStub:NewLibrary(LIBXML, LIBXML_MINOR)
if not LibXML then return end

function LibXML.parseargs(s, l)
  local arg = {};
  string.gsub(s, "([%w%-]+)=([\"'])(.-)%2", function (w, _, a) arg[w] = a end);
  if l then string.gsub(s, "([%w%-]+)=([%w%.%-]+)", function (w, a) arg[w] = a end); end
  return arg;
end
function LibXML.at(s,e)
	local l = 1;
	for n = 1, e do if s:sub(n,n)=="\n" or s:sub(n,n+1)=="|n" then l=l+1; end end
	return l
end

--- XML Import function
-- Imports the given XML string and returns a table structure containing the XML elements
-- @class function
-- @name LibXML:Import
-- @param xml_data The XML file (string) to be parsed.
-- @param loose_mode If set to <code>true</code>, LibXML will permit a looser syntax (optional)
-- @return <code>nil</code> if the XML is invalid, otherwise a table struct containing the XML elements
-- @return A message of any error encounted while parsing the XML data.
-- @usage
-- local xml_data = [[ <xml> <img src="some image URL" /> <p>Text goes here!<br/>And here!</p></xml> ]];
-- local libxml = LibStub("LibXML-1.0);
-- local struct, errmsg = libxml:Import(xml_data);
-- if ( errmsg ) then error(errmsg)
-- else
--     for i = 1, #struct do
--         local element = struct[i]
--         if ( element.class == "img" ) then ImageStuff(element.args.src) end
--         if ( element.class == "p" ) then
--             for j = 1, #element do
--                 local subElement = element[i]
--                 if ( type(subElement) == "string" ) then print(subElement) end
--                 if ( type(subElement) == "table" and subElement.class == "br" ) then
--                     print("We have a line break!\n");
--                 end
--             end
--         end
--     end
-- end
function LibXML:Import(s, loose)
  local stack = {};
  local top = {};
  table.insert(stack, top);
  local ni,c,class,args,empty;
  local i, j = 1, 1;
  while true do
    ni,j,c,class,args, empty = string.find(s, "<(%/?)([%w:]+)(.-)(%/?)>", i);
    if not ni then break end
    local text = string.sub(s, i, ni-1);
    if not string.find(text, "^%s*$") then
      table.insert(top, text);
    end
    if empty == "/" then  -- empty element
      table.insert(top, {class=class, args=LibXML.parseargs(args, l), empty=true});
    elseif c == "" then   -- start tag
      top = {class=class, args=LibXML.parseargs(args, l),l=LibXML.at(s,ni)};
      table.insert(stack, top);   -- new level
    else  -- end tag
      local toclose = table.remove(stack);  -- pop the top and get the opening tag
      top = stack[#stack];
      if #stack < 1 then
        return nil, ("%s: No opening tag found for <%s> at line %u!"):format(LIBXML, class, LibXML.at(s,ni));
      end
      if toclose.class ~= class then
        return nil, ("%s: Trying to close <%s> (at line %u) with </%s> (at line %u)!"):format(LIBXML, toclose.class, toclose.l, class, LibXML.at(s,ni));
      end
      table.insert(top, toclose);
    end
    i = j+1;
  end
  local text = string.sub(s, i);
  if not string.find(text, "^%s*$") then
    table.insert(stack[#stack], text);
  end
  if #stack > 1 then
    return nil, ("%s: XML data ended without closing <%s> (at line %u)!"):format(LIBXML, stack[#stack].class, stack[#stack].l);
  end
  return stack[1];
end
